@@ -1,6 +1,6 @@
 #! /bin/sh
 # check script for GNU ddrescue - Data recovery tool
-# Copyright (C) 2009-2017 Antonio Diaz Diaz.
+# Copyright (C) 2009-2018 Antonio Diaz Diaz.
 #
 # This script is free software: you have unlimited permission
 # to copy, distribute and modify it.
@@ -22,6 +22,7 @@ if [ -d tmp ] ; then rm -rf tmp ; fi
 mkdir tmp
 cd "${objdir}"/tmp || framework_failure
 
+fox="${testdir}"/fox
 in="${testdir}"/test.txt
 in1="${testdir}"/test1.txt
 in2="${testdir}"/test2.txt
@@ -54,12 +55,30 @@ printf "testing ddrescue-%s..." "$2"
 [ $? = 1 ] || test_failed $LINENO
 "${DDRESCUE}" -q -a 1K ${in} out
 [ $? = 1 ] || test_failed $LINENO
+"${DDRESCUE}" -q -a 1BB ${in} out
+[ $? = 1 ] || test_failed $LINENO
+"${DDRESCUE}" -q -a 1Bi ${in} out
+[ $? = 1 ] || test_failed $LINENO
+"${DDRESCUE}" -q -a 1Bk ${in} out
+[ $? = 1 ] || test_failed $LINENO
+"${DDRESCUE}" -q -a 1Bs ${in} out
+[ $? = 1 ] || test_failed $LINENO
+"${DDRESCUE}" -q -a 1iB ${in} out
+[ $? = 1 ] || test_failed $LINENO
+"${DDRESCUE}" -q -a 1ii ${in} out
+[ $? = 1 ] || test_failed $LINENO
+"${DDRESCUE}" -q -a 1ik ${in} out
+[ $? = 1 ] || test_failed $LINENO
+"${DDRESCUE}" -q -a 1is ${in} out
+[ $? = 1 ] || test_failed $LINENO
 "${DDRESCUE}" -q -a 1kk ${in} out
 [ $? = 1 ] || test_failed $LINENO
-"${DDRESCUE}" -q -a 1sk ${in} out
+"${DDRESCUE}" -q -a 1sB ${in} out
 [ $? = 1 ] || test_failed $LINENO
 "${DDRESCUE}" -q -a 1si ${in} out
 [ $? = 1 ] || test_failed $LINENO
+"${DDRESCUE}" -q -a 1sk ${in} out
+[ $? = 1 ] || test_failed $LINENO
 "${DDRESCUE}" -q -a 1ss ${in} out
 [ $? = 1 ] || test_failed $LINENO
 "${DDRESCUE}" -q -F- ${in} out
@@ -68,6 +87,8 @@ printf "testing ddrescue-%s..." "$2"
 [ $? = 1 ] || test_failed $LINENO
 "${DDRESCUE}" -q -F- --ask ${in} out mapfile
 [ $? = 1 ] || test_failed $LINENO
+"${DDRESCUE}" -q -F- --same-file ${in} out mapfile
+[ $? = 1 ] || test_failed $LINENO
 "${DDRESCUE}" -q -G --ask ${in} out mapfile
 [ $? = 1 ] || test_failed $LINENO
 "${DDRESCUE}" -q -G ${in} out
@@ -116,6 +137,8 @@ printf "testing ddrescue-%s..." "$2"
 [ $? = 1 ] || test_failed $LINENO
 "${DDRESCUE}" -q --mapfile-interval=30,60s, ${in} out
 [ $? = 1 ] || test_failed $LINENO
+"${DDRESCUE}" -q --same-file -t ${in} out
+[ $? = 1 ] || test_failed $LINENO
 
 rm -f mapfile
 "${DDRESCUE}" -q -t -p -J -b1024 -i15kB ${in} out mapfile || test_failed $LINENO
@@ -228,6 +251,19 @@ cat ${in2} > out || framework_failure
 "${DDRESCUE}" -q -R -T1.5d copy out mapfile || test_failed $LINENO
 cmp ${in} out || test_failed $LINENO
 
+"${DDRESCUE}" -q --same-file out out || test_failed $LINENO
+cmp ${in} out || test_failed $LINENO
+
+"${DDRESCUE}" -q -t ${in} out || test_failed $LINENO
+"${DDRESCUE}" -q --same-file -o 72776 out out || test_failed $LINENO
+cat ${in} ${in} | cmp out - || test_failed $LINENO
+
+rm -f out
+"${DDRESCUE}" -q ${in} out || test_failed $LINENO
+"${DDRESCUE}" -q --same-file -o 145552 -S out out || test_failed $LINENO
+"${DDRESCUE}" -q --same-file -i 145552 -o 72776 out out || test_failed $LINENO
+cat ${in} ${in} ${in} | cmp out - || test_failed $LINENO
+
 printf "\ntesting ddrescuelog-%s..." "$2"
 
 "${DDRESCUELOG}" -q mapfile
@@ -240,6 +276,8 @@ printf "\ntesting ddrescuelog-%s..." "$2"
 [ $? = 1 ] || test_failed $LINENO
 "${DDRESCUELOG}" -q -m ${map2i} -t mapfile
 [ $? = 2 ] || test_failed $LINENO
+"${DDRESCUELOG}" -q --shift -i20 mapfile
+[ $? = 1 ] || test_failed $LINENO
 
 "${DDRESCUELOG}" -a '?,+' -i3072 - < ${map1} > mapfile
 "${DDRESCUELOG}" -D - < mapfile
@@ -259,14 +297,14 @@ cmp out copy || test_failed $LINENO
 "${DDRESCUELOG}" -q -p ${map1} mapfile
 [ $? = 1 ] || test_failed $LINENO
 "${DDRESCUELOG}" -P ${map1} mapfile || test_failed $LINENO
-"${DDRESCUELOG}" -b2048 -s72776 -f -c?+ mapfile < out || test_failed $LINENO
+"${DDRESCUELOG}" -b2048 -s72776 -f '-c?+' mapfile < out || test_failed $LINENO
 "${DDRESCUELOG}" -p ${map2} - < mapfile || test_failed $LINENO
-"${DDRESCUELOG}" -b2048 -f -c?+ mapfile < out || test_failed $LINENO
+"${DDRESCUELOG}" -b2048 -f '-c?+' mapfile < out || test_failed $LINENO
 "${DDRESCUELOG}" -s72776 -p ${map2} mapfile || test_failed $LINENO
 "${DDRESCUELOG}" -q -s72777 -p ${map2} mapfile
 [ $? = 1 ] || test_failed $LINENO
 
-printf "10\n12\n14\n16\n" | "${DDRESCUELOG}" -b2048 -f -c+? mapfile ||
+printf "10\n12\n14\n16\n" | "${DDRESCUELOG}" -b2048 -f '-c+?' mapfile ||
 	test_failed $LINENO
 "${DDRESCUELOG}" -q -p mapfile ${map1}
 [ $? = 1 ] || test_failed $LINENO
@@ -295,7 +333,7 @@ cat ${map2} > mapfile || framework_failure
 "${DDRESCUELOG}" -b2048 -l+ ${map1} > out || test_failed $LINENO
 printf "0\n2\n4\n6\n8\n10\n12\n14\n16\n18\n20\n22\n24\n26\n28\n30\n32\n34\n" > copy || framework_failure
 cmp out copy || test_failed $LINENO
-"${DDRESCUELOG}" -b2048 -l?- ${map1} > out || test_failed $LINENO
+"${DDRESCUELOG}" -b2048 '-l?-' ${map1} > out || test_failed $LINENO
 printf "1\n3\n5\n7\n9\n11\n13\n15\n17\n19\n21\n23\n25\n27\n29\n31\n33\n35\n" > copy || framework_failure
 cmp out copy || test_failed $LINENO
 "${DDRESCUELOG}" -b2048 -l+ -i0x1800 -o0 -s0x4000 ${map1} > out ||
@@ -307,7 +345,7 @@ cmp out copy || test_failed $LINENO
 "${DDRESCUELOG}" -b2048 -l+ mapfile > out || test_failed $LINENO
 printf "0\n2\n4\n6\n8\n10\n12\n14\n16\n18\n20\n22\n24\n26\n28\n30\n32\n34\n" > copy || framework_failure
 cmp out copy || test_failed $LINENO
-"${DDRESCUELOG}" -b2048 -l?- mapfile > out || test_failed $LINENO
+"${DDRESCUELOG}" -b2048 '-l?-' mapfile > out || test_failed $LINENO
 printf "1\n3\n5\n7\n9\n11\n13\n15\n17\n19\n21\n23\n25\n27\n29\n31\n33\n35\n" > copy || framework_failure
 cmp out copy || test_failed $LINENO
 "${DDRESCUELOG}" -b2048 -l+ -i2048 -o0 -s0x4000 mapfile > out ||
@@ -406,7 +444,7 @@ done
 "${DDRESCUELOG}" -b2048 -l+ ${map1} > out || test_failed $LINENO
 "${DDRESCUELOG}" -y ${map1} - < ${map2} > mapfile || test_failed $LINENO
 "${DDRESCUELOG}" -P ${blank} mapfile || test_failed $LINENO
-"${DDRESCUELOG}" -b2048 -l? mapfile > copy || test_failed $LINENO
+"${DDRESCUELOG}" -b2048 '-l?' mapfile > copy || test_failed $LINENO
 cmp out copy || test_failed $LINENO
 "${DDRESCUELOG}" -y ${map2} ${map1} > mapfile || test_failed $LINENO
 "${DDRESCUELOG}" -P ${blank} mapfile || test_failed $LINENO
@@ -508,6 +546,38 @@ for i in ${map1} ${map2} ${map3} ${map4} ${map5} ; do
 	done
 done
 
+"${DDRESCUELOG}" --shift -o0x800 -s0x11448 ${map1} > out || test_failed $LINENO
+"${DDRESCUELOG}" -p ${map2} out || test_failed $LINENO
+
+"${DDRESCUELOG}" --shift -o0x400 ${map1} > out || test_failed $LINENO
+"${DDRESCUELOG}" --shift -o0x400 -s0x11848 out > copy || test_failed $LINENO
+"${DDRESCUELOG}" -p ${map2} copy || test_failed $LINENO
+
+"${DDRESCUELOG}" --shift -o0x400 ${map2} > out || test_failed $LINENO
+"${DDRESCUELOG}" --shift -i0x400 -o0 out > copy || test_failed $LINENO
+"${DDRESCUELOG}" -p ${map2} copy || test_failed $LINENO
+
+"${DDRESCUELOG}" --shift -o0x900 ${map1} > copy || test_failed $LINENO
+"${DDRESCUELOG}" --shift -i0x300 -o0 copy > out || test_failed $LINENO
+"${DDRESCUELOG}" --shift -i0x600 -o0 out > copy || test_failed $LINENO
+"${DDRESCUELOG}" -p ${map1} copy || test_failed $LINENO
+
+"${DDRESCUELOG}" --shift -i0x800 -o0 ${map2} > copy || test_failed $LINENO
+"${DDRESCUELOG}" --shift -o0x488 copy > out || test_failed $LINENO
+"${DDRESCUELOG}" --shift -o0x378 out > copy || test_failed $LINENO
+"${DDRESCUELOG}" -p ${map2} copy || test_failed $LINENO
+
+printf "\ntesting combined rescue..."
+
+rm -f mapfile
+"${DDRESCUE}" -q ${in} out mapfile		# rescue partition
+"${DDRESCUELOG}" --shift -o45 mapfile > shifted_mapfile
+"${DDRESCUE}" -q --same-file -o45 -s72776 --reverse out out
+"${DDRESCUE}" -q -C ${fox} out shifted_mapfile	# rescue rest of drive
+cat ${fox} ${in} | cmp out - || test_failed $LINENO
+"${DDRESCUELOG}" -d mapfile || test_failed $LINENO
+"${DDRESCUELOG}" -d shifted_mapfile || test_failed $LINENO
+
 echo
 if [ ${fail} = 0 ] ; then
 	echo "tests completed successfully."
